!MNH_LIC Copyright 1994-2022 CNRS, Meteo-France and Universite Paul Sabatier
!MNH_LIC This is part of the Meso-NH software governed by the CeCILL-C licence
!MNH_LIC version 1. See LICENSE, CeCILL-C_V1-en.txt and CeCILL-C_V1-fr.txt
!MNH_LIC for details. version 1.
!-----------------------------------------------------------------
MODULE MODI_ICE4_COMPUTE_PDF
INTERFACE
SUBROUTINE ICE4_COMPUTE_PDF(HSUBG_AUCV_RC, HSUBG_AUCV_RI, HSUBG_PR_PDF, &
                            PRHODREF, PRCT, PRIT, PCF, PT, PSIGMA_RC,&
                            PHLC_HCF, PHLC_LCF, PHLC_HRC, PHLC_LRC, &
                            PHLI_HCF, PHLI_LCF, PHLI_HRI, PHLI_LRI, PRF)
IMPLICIT NONE
CHARACTER(LEN=4),   INTENT(IN)  :: HSUBG_AUCV_RC     ! Kind of Subgrid autoconversion method
CHARACTER(LEN=80),  INTENT(IN)  :: HSUBG_AUCV_RI     ! Kind of Subgrid autoconversion method
CHARACTER(LEN=80),  INTENT(IN)  :: HSUBG_PR_PDF   ! pdf for subgrid precipitation
REAL, DIMENSION(:), INTENT(IN)  :: PRHODREF   ! Reference density
REAL, DIMENSION(:), INTENT(IN)  :: PRCT       ! Cloud water m.r. at t
REAL, DIMENSION(:), INTENT(IN)  :: PRIT       ! Ice Crystal m.r. at t
REAL, DIMENSION(:), INTENT(IN)  :: PCF        ! Cloud fraction
REAL, DIMENSION(:), INTENT(IN)  :: PT         ! Temperature
REAL, DIMENSION(:), INTENT(IN)  :: PSIGMA_RC  ! Standard deviation of rc at time t
REAL, DIMENSION(:), INTENT(OUT) :: PHLC_HCF   ! HLCLOUDS : fraction of High Cloud Fraction in grid
REAL, DIMENSION(:), INTENT(OUT) :: PHLC_LCF   ! HLCLOUDS : fraction of Low  Cloud Fraction in grid
                                              !    note that PCF = PHLC_HCF + PHLC_LCF
REAL, DIMENSION(:), INTENT(OUT) :: PHLC_HRC   ! HLCLOUDS : LWC that is High LWC in grid
REAL, DIMENSION(:), INTENT(OUT) :: PHLC_LRC   ! HLCLOUDS : LWC that is Low  LWC in grid
                                              !    note that PRC = PHLC_HRC + PHLC_LRC
REAL, DIMENSION(:), INTENT(INOUT) :: PHLI_HCF   !
REAL, DIMENSION(:), INTENT(INOUT) :: PHLI_LCF
REAL, DIMENSION(:), INTENT(INOUT) :: PHLI_HRI   !
REAL, DIMENSION(:), INTENT(INOUT) :: PHLI_LRI   !
REAL, DIMENSION(:), INTENT(OUT) :: PRF        ! Rain fraction
END SUBROUTINE ICE4_COMPUTE_PDF
END INTERFACE
END MODULE MODI_ICE4_COMPUTE_PDF
SUBROUTINE ICE4_COMPUTE_PDF(HSUBG_AUCV_RC, HSUBG_AUCV_RI, HSUBG_PR_PDF, &
                            PRHODREF, PRCT, PRIT, PCF, PT, PSIGMA_RC,&
                            PHLC_HCF, PHLC_LCF, PHLC_HRC, PHLC_LRC, &
                            PHLI_HCF, PHLI_LCF, PHLI_HRI, PHLI_LRI, PRF)
!!
!!**  PURPOSE
!!    -------
!!      Computes the pdf used to split cloud into high and low content parts
!!
!!    AUTHOR
!!    ------
!!      S. Riette from the plitting of rain_ice source code (nov. 2014)
!!
!!    MODIFICATIONS
!!    -------------
!!
!
!
!*      0. DECLARATIONS
!          ------------
!
!
USE MODD_RAIN_ICE_DESCR, ONLY: XRTMIN
USE MODD_RAIN_ICE_PARAM, ONLY: XCRIAUTC,XBCRIAUTI,XACRIAUTI,XCRIAUTI
USE MODD_CST, ONLY : XTT
!
#ifdef MNH_OPENACC
USE MODE_MNH_ZWORK,      ONLY: MNH_MEM_GET, MNH_MEM_POSITION_PIN, MNH_MEM_RELEASE
#endif
USE MODE_MSG
!
#ifdef MNH_BITREP
USE MODI_BITREP
#endif
!
USE MODE_MPPDB
!
IMPLICIT NONE
!
!*       0.1   Declarations of dummy arguments :
!
CHARACTER(LEN=4),   INTENT(IN)  :: HSUBG_AUCV_RC     ! Kind of Subgrid autoconversion method
CHARACTER(LEN=80),  INTENT(IN)  :: HSUBG_AUCV_RI     ! Kind of Subgrid autoconversion method
CHARACTER(LEN=80),  INTENT(IN)  :: HSUBG_PR_PDF   ! pdf for subgrid precipitation
REAL, DIMENSION(:), INTENT(IN)  :: PRHODREF   ! Reference density
REAL, DIMENSION(:), INTENT(IN)  :: PRCT       ! Cloud water m.r. at t
REAL, DIMENSION(:), INTENT(IN)  :: PRIT       ! Ice Crystal m.r. at t
REAL, DIMENSION(:), INTENT(IN)  :: PCF        ! Cloud fraction
REAL, DIMENSION(:), INTENT(IN)  :: PT         ! Temperature
REAL, DIMENSION(:), INTENT(IN)  :: PSIGMA_RC  ! Standard deviation of rc at time t
REAL, DIMENSION(:), INTENT(OUT) :: PHLC_HCF   ! HLCLOUDS : fraction of High Cloud Fraction in grid
REAL, DIMENSION(:), INTENT(OUT) :: PHLC_LCF   ! HLCLOUDS : fraction of Low  Cloud Fraction in grid
                                              !    note that PCF = PHLC_HCF + PHLC_LCF
REAL, DIMENSION(:), INTENT(OUT) :: PHLC_HRC   ! HLCLOUDS : LWC that is High LWC in grid
REAL, DIMENSION(:), INTENT(OUT) :: PHLC_LRC   ! HLCLOUDS : LWC that is Low  LWC in grid
                                              !    note that PRC = PHLC_HRC + PHLC_LRC
REAL, DIMENSION(:), INTENT(INOUT) :: PHLI_HCF   !
REAL, DIMENSION(:), INTENT(INOUT) :: PHLI_LCF
REAL, DIMENSION(:), INTENT(INOUT) :: PHLI_HRI   !
REAL, DIMENSION(:), INTENT(INOUT) :: PHLI_LRI   !
REAL, DIMENSION(:), INTENT(OUT) :: PRF        ! Rain fraction
!
!*       0.2  declaration of local variables
!
INTEGER                            :: ISIZE
INTEGER                            :: JI
#ifndef MNH_OPENACC
LOGICAL, DIMENSION(:), allocatable :: GWORK, GWORK2, GWORK3
#else
LOGICAL, DIMENSION(:), pointer,contiguous :: GWORK, GWORK2, GWORK3
#endif
REAL                               :: ZCOEFFRCM
#ifndef MNH_OPENACC
REAL,    DIMENSION(:), allocatable :: ZRCRAUTC,      & !RC value to begin rain formation =XCRIAUTC/RHODREF
#else
REAL,    DIMENSION(:), pointer,contiguous :: ZRCRAUTC,      & !RC value to begin rain formation =XCRIAUTC/RHODREF
#endif
                                      ZCRIAUTI,      & !RI value to begin snow formation
                                      ZHLC_RCMAX,    & !HLCLOUDS : maximum value for RC in distribution
                                      ZHLC_LRCLOCAL, & !HLCLOUDS : LWC that is Low  LWC local in LCF
                                      ZHLC_HRCLOCAL, & !HLCLOUDS : LWC that is High LWC local in HCF
                                                       !    note that ZRC/CF = ZHLC_HRCLOCAL+ ZHLC_LRCLOCAL
                                                       !                     = PHLC_HRC/HCF+ PHLC_LRC/LCF
                                      ZSUMRC, ZSUMRI
! 
INTEGER :: IHSUBG_PR_PDF ! temporary variable for OpenCC character limitation (Cray CCE)
!
!-------------------------------------------------------------------------------

!$acc data present(PRHODREF,PRCT,PRIT,PCF,PT,PSIGMA_RC, &
!$acc&                PHLC_HCF,PHLC_LCF,PHLC_HRC,PHLC_LRC,PHLI_HCF,PHLI_LCF,PHLI_HRI,PHLI_LRI,PRF)

IF (MPPDB_INITIALIZED) THEN
  !Check all IN arrays
  CALL MPPDB_CHECK1D(PRHODREF,"ICE4_COMPUTE_PDF beg:PRHODREF",PRECISION)
  CALL MPPDB_CHECK1D(PRCT,"ICE4_COMPUTE_PDF beg:PRCT",PRECISION)
  CALL MPPDB_CHECK1D(PRIT,"ICE4_COMPUTE_PDF beg:PRIT",PRECISION)
  CALL MPPDB_CHECK1D(PCF,"ICE4_COMPUTE_PDF beg:PCF",PRECISION)
  CALL MPPDB_CHECK1D(PT,"ICE4_COMPUTE_PDF beg:PT",PRECISION)
  CALL MPPDB_CHECK1D(PSIGMA_RC,"ICE4_COMPUTE_PDF beg:PSIGMA_RC",PRECISION)
  !Check all INOUT arrays
  CALL MPPDB_CHECK1D(PHLI_HCF,"ICE4_COMPUTE_PDF beg:PHLI_HCF",PRECISION)
  CALL MPPDB_CHECK1D(PHLI_LCF,"ICE4_COMPUTE_PDF beg:PHLI_LCF",PRECISION)
  CALL MPPDB_CHECK1D(PHLI_HRI,"ICE4_COMPUTE_PDF beg:PHLI_HRI",PRECISION)
  CALL MPPDB_CHECK1D(PHLI_LRI,"ICE4_COMPUTE_PDF beg:PHLI_LRI",PRECISION)
END IF

isize = size( prhodref )

#ifndef MNH_OPENACC
allocate( gwork (isize ) )
allocate( gwork2(isize ) )
allocate( gwork3(isize ) )

allocate( zrcrautc     (isize ) )
allocate( zcriauti     (isize ) )
allocate( zhlc_rcmax   (isize ) )
allocate( zhlc_lrclocal(isize ) )
allocate( zhlc_hrclocal(isize ) )
allocate( ZSUMRC       (isize ) )
allocate( ZSUMRI       (isize ) )
#else
!Pin positions in the pools of MNH memory
CALL MNH_MEM_POSITION_PIN( 'ICE4_COMPUTE_PDF' )

CALL MNH_MEM_GET( gwork,  isize )
CALL MNH_MEM_GET( gwork2, isize )
CALL MNH_MEM_GET( gwork3, isize )

CALL MNH_MEM_GET( zrcrautc,      isize )
CALL MNH_MEM_GET( zcriauti,      isize )
CALL MNH_MEM_GET( zhlc_rcmax,    isize )
CALL MNH_MEM_GET( zhlc_lrclocal, isize )
CALL MNH_MEM_GET( zhlc_hrclocal, isize )
CALL MNH_MEM_GET( ZSUMRC,        isize )
CALL MNH_MEM_GET( ZSUMRI,        isize )

!$acc data present( GWORK, GWORK2, GWORK3, ZRCRAUTC, ZCRIAUTI, ZHLC_RCMAX, ZHLC_LRCLOCAL, ZHLC_HRCLOCAL, ZSUMRC, ZSUMRI )
#endif

!Cloud water split between high and low content part is done according to autoconversion option
!$acc kernels
ZRCRAUTC(:)=XCRIAUTC/PRHODREF(:) ! Autoconversion rc threshold
!$acc end kernels
IF(HSUBG_AUCV_RC=='NONE') THEN
!$acc kernels
  DO JI = 1, ISIZE
    IF (PRCT(JI)>ZRCRAUTC(JI)) THEN
      PHLC_HCF(JI)=1.
      PHLC_LCF(JI)=0.
      PHLC_HRC(JI)=PRCT(JI)
      PHLC_LRC(JI)=0.
    ELSE IF (PRCT(JI)>XRTMIN(2)) THEN
      PHLC_HCF(JI)=0.
      PHLC_LCF(JI)=1.
      PHLC_HRC(JI)=0.
      PHLC_LRC(JI)=PRCT(JI)
    ELSE
      PHLC_HCF(JI)=0.
      PHLC_LCF(JI)=0.
      PHLC_HRC(JI)=0.
      PHLC_LRC(JI)=0.
    END IF
  END DO
!$acc end kernels

ELSEIF(HSUBG_AUCV_RC=='CLFR') THEN
!$acc kernels present( GWORK, GWORK2, GWORK3 )
  !Cloud water is only in the cloudy part and entirely in low or high part
  GWORK3(:) = PCF(:) > 0.
  GWORK(:)  = GWORK3(:) .AND. PRCT(:)>ZRCRAUTC(:)*PCF(:)
  GWORK2(:) = GWORK3(:) .AND. PRCT(:)>XRTMIN(2)
  WHERE(GWORK(:))
    PHLC_HCF(:)=PCF(:)
    PHLC_LCF(:)=0.
    PHLC_HRC(:)=PRCT(:)
    PHLC_LRC(:)=0.
  ELSEWHERE(GWORK2(:))
    PHLC_HCF(:)=0.
    PHLC_LCF(:)=PCF(:)
    PHLC_HRC(:)=0.0
    PHLC_LRC(:)=PRCT(:)
  ELSEWHERE
    PHLC_HCF(:)=0.
    PHLC_LCF(:)=0.
    PHLC_HRC(:)=0.
    PHLC_LRC(:)=0.
  END WHERE
!$acc end kernels
ELSEIF(HSUBG_AUCV_RC=='ADJU') THEN
!$acc kernels present( GWORK )
  ZSUMRC(:)=PHLC_LRC(:)+PHLC_HRC(:)
  !$mnh_do_concurrent( JI = 1 : ISIZE )
    GWORK(JI) = ZSUMRC(JI) > 0.
  !$mnh_end_do()
  WHERE(GWORK(:))
    PHLC_LRC(:)=PHLC_LRC(:)*PRCT(:)/ZSUMRC(:)
    PHLC_HRC(:)=PHLC_HRC(:)*PRCT(:)/ZSUMRC(:)
  ELSEWHERE
    PHLC_LRC(:)=0.
    PHLC_HRC(:)=0.
  ENDWHERE 
!$acc end kernels
ELSEIF(HSUBG_AUCV_RC=='PDF ') THEN
  !Cloud water is split between high and low part according to a PDF
  !    'HLCRECTPDF'    : rectangular PDF form
  !    'HLCTRIANGPDF'  : triangular PDF form
  !    'HLCQUADRAPDF'  : second order quadratic PDF form
  !    'HLCISOTRIPDF'  : isocele triangular PDF
  !    'SIGM'          : Redelsperger and Sommeria (1986)
  IF(HSUBG_PR_PDF=='SIGM') THEN
!$acc kernels present( GWORK, GWORK2, GWORK3 )
    ! Redelsperger and Sommeria (1986) but organised according to Turner (2011, 2012)
    !$mnh_do_concurrent( JI = 1 : ISIZE )
      GWORK(JI)  = PRCT(JI)>  ZRCRAUTC(JI)+PSIGMA_RC(JI)
      GWORK2(JI) = PRCT(JI)> (ZRCRAUTC(JI)-PSIGMA_RC(JI)) .AND. &
                   PRCT(JI)<=(ZRCRAUTC(JI)+PSIGMA_RC(JI))
      GWORK3(JI) = PRCT(JI)>XRTMIN(2) .AND. PCF(JI)>0.
    !$mnh_end_do()
    WHERE (GWORK(:))
      PHLC_HCF(:)=1.
      PHLC_LCF(:)=0.
      PHLC_HRC(:)=PRCT(:)
      PHLC_LRC(:)=0.
    ELSEWHERE(GWORK2(:))
      PHLC_HCF(:)=(PRCT(:)+PSIGMA_RC(:)-ZRCRAUTC(:))/ &
                  &(2.*PSIGMA_RC(:))
      PHLC_LCF(:)=MAX(0., PCF(:)-PHLC_HCF(:))
      PHLC_HRC(:)=(PRCT(:)+PSIGMA_RC(:)-ZRCRAUTC(:))* &
                  &(PRCT(:)+PSIGMA_RC(:)+ZRCRAUTC(:))/ &
                  &(4.*PSIGMA_RC(:))
      PHLC_LRC(:)=MAX(0., PRCT(:)-PHLC_HRC(:))
    ELSEWHERE(GWORK3(:))
      PHLC_HCF(:)=0.
      PHLC_LCF(:)=PCF(:)
      PHLC_HRC(:)=0.
      PHLC_LRC(:)=PRCT(:)
    ELSEWHERE
      PHLC_HCF(:)=0.
      PHLC_LCF(:)=0.
      PHLC_HRC(:)=0.
      PHLC_LRC(:)=0.
    END WHERE
   ! Turner (2011, 2012)
!$acc end kernels
  ELSEIF(HSUBG_PR_PDF=='HLCRECTPDF' .OR. HSUBG_PR_PDF=='HLCISOTRIPDF' .OR. &
         &HSUBG_PR_PDF=='HLCTRIANGPDF' .OR. HSUBG_PR_PDF=='HLCQUADRAPDF') THEN
    ! Calculate maximum value r_cM from PDF forms
    IF(HSUBG_PR_PDF=='HLCRECTPDF' .OR. HSUBG_PR_PDF=='HLCISOTRIPDF') THEN
      ZCOEFFRCM=2.
    ELSE IF(HSUBG_PR_PDF=='HLCTRIANGPDF') THEN
      ZCOEFFRCM=3.
    ELSE IF(HSUBG_PR_PDF=='HLCQUADRAPDF') THEN
      ZCOEFFRCM=4.
    END IF
!$acc update device(ZCOEFFRCM)

SELECT CASE( HSUBG_PR_PDF )
  CASE ( 'HLCRECTPDF' )
     IHSUBG_PR_PDF = 1
  CASE ( 'HLCTRIANGPDF' )
     IHSUBG_PR_PDF = 2
  CASE ( 'HLCQUADRAPDF' )
     IHSUBG_PR_PDF = 3
  CASE ( 'HLCISOTRIPDF' )
     IHSUBG_PR_PDF = 4
  CASE DEFAULT
     IHSUBG_PR_PDF = 0
END SELECT

!$acc kernels
    !$mnh_do_concurrent( JI = 1 : ISIZE )
      IF ( PRCT(JI)>0. .AND. PCF(JI)>0. ) ZHLC_RCMAX(JI) = ZCOEFFRCM * PRCT(JI) / PCF(JI)
    !$mnh_end_do()
!$acc end kernels
    ! Split available water and cloud fraction in two parts
    ! Calculate local mean values int he low and high parts for the 3 PDF forms:
!$acc kernels present( GWORK )
    !$mnh_do_concurrent( JI = 1 : ISIZE )
      GWORK(JI) = PRCT(JI)>0. .AND. PCF(JI)>0. .AND. ZHLC_RCMAX(JI)>ZRCRAUTC(JI)
    !$mnh_end_do()
!$acc end kernels
    !IF(HSUBG_PR_PDF=='HLCRECTPDF') THEN
    IF (IHSUBG_PR_PDF==1) THEN
!$acc kernels present( GWORK )
      WHERE(GWORK(:))
        ZHLC_LRCLOCAL(:)=0.5*ZRCRAUTC(:)
        ZHLC_HRCLOCAL(:)=( ZHLC_RCMAX(:) + ZRCRAUTC(:))/2.0
      END WHERE
!$acc end kernels
    !ELSE IF(HSUBG_PR_PDF=='HLCTRIANGPDF') THEN
    ELSE IF (IHSUBG_PR_PDF==2) THEN
!$acc kernels present( GWORK )
      WHERE(GWORK(:))
        ZHLC_LRCLOCAL(:)=( ZRCRAUTC(:) *(3.0 * ZHLC_RCMAX(:) - 2.0 * ZRCRAUTC(:) ) ) &
                        / (3.0 * (2.0 * ZHLC_RCMAX(:) - ZRCRAUTC(:)  ) )
        ZHLC_HRCLOCAL(:)=(ZHLC_RCMAX(:) + 2.0*ZRCRAUTC(:)) / 3.0
      END WHERE
!$acc end kernels
    !ELSE IF(HSUBG_PR_PDF=='HLCQUADRAPDF') THEN
    ELSE IF (IHSUBG_PR_PDF==3) THEN
!$acc kernels present( GWORK )
      !$mnh_do_concurrent( JI = 1 : ISIZE )
        IF ( GWORK(JI) ) THEN
#ifndef MNH_BITREP
          ZHLC_LRCLOCAL(JI)=(3.0 *ZRCRAUTC(JI)**3 - 8.0 *ZRCRAUTC(JI)**2 * ZHLC_RCMAX(JI) &
                           + 6.0*ZRCRAUTC(JI) *ZHLC_RCMAX(JI)**2 ) &
                           / &
                           (4.0* ZRCRAUTC(JI)**2 -12.0*ZRCRAUTC(JI) *ZHLC_RCMAX(JI) &
                           + 12.0 * ZHLC_RCMAX(JI)**2 )
#else
          ZHLC_LRCLOCAL(JI)=(3.0 *BR_P3(ZRCRAUTC(JI)) - 8.0 *BR_P2(ZRCRAUTC(JI)) * ZHLC_RCMAX(JI) &
                           + 6.0*ZRCRAUTC(JI) *BR_P2(ZHLC_RCMAX(JI)) ) &
                           / &
                           (4.0* BR_P2(ZRCRAUTC(JI)) -12.0*ZRCRAUTC(JI) *ZHLC_RCMAX(JI) &
                           + 12.0 * BR_P2(ZHLC_RCMAX(JI)) )
#endif
          ZHLC_HRCLOCAL(JI)=(ZHLC_RCMAX(JI) + 3.0*ZRCRAUTC(JI))/4.0
        END IF
      !$mnh_end_do()
!$acc end kernels
    !ELSE IF(HSUBG_PR_PDF=='HLCISOTRIPDF') THEN
    ELSE IF (IHSUBG_PR_PDF==4) THEN
!$acc kernels present( GWORK, GWORK2 )
      !$mnh_do_concurrent( JI = 1 : ISIZE )
        IF ( GWORK(JI) ) THEN
          IF ( PRCT(JI) <= ZRCRAUTC(JI)*PCF(JI) ) THEN
#ifndef MNH_BITREP
            ZHLC_LRCLOCAL(JI)=( (ZHLC_RCMAX(JI))**3 &
                             -(12.0 * (ZHLC_RCMAX(JI))*(ZRCRAUTC(JI))**2) &
                             +(8.0 * ZRCRAUTC(JI)**3) ) &
                             /( (6.0 * (ZHLC_RCMAX(JI))**2) &
                             -(24.0 * (ZHLC_RCMAX(JI)) * ZRCRAUTC(JI)) &
                             +(12.0 * ZRCRAUTC(JI)**2) )
#else
            ZHLC_LRCLOCAL(JI)=( BR_P3(ZHLC_RCMAX(JI)) &
                             -(12.0 * (ZHLC_RCMAX(JI))*BR_P2(ZRCRAUTC(JI))) &
                             +(8.0 * BR_P3(ZRCRAUTC(JI))) ) &
                             /( (6.0 * BR_P2(ZHLC_RCMAX(JI))) &
                             -(24.0 * (ZHLC_RCMAX(JI)) * ZRCRAUTC(JI)) &
                             +(12.0 * BR_P2(ZRCRAUTC(JI))) )
#endif
            ZHLC_HRCLOCAL(JI)=( ZHLC_RCMAX(JI) + 2.0 * ZRCRAUTC(JI) )/3.0
          ELSE
            ZHLC_LRCLOCAL(JI)=(2.0/3.0) * ZRCRAUTC(JI)
#ifndef MNH_BITREP
            ZHLC_HRCLOCAL(JI)=(3.0*ZHLC_RCMAX(JI)**3 - 8.0*ZRCRAUTC(JI)**3) &
                             / (6.0 * ZHLC_RCMAX(JI)**2 - 12.0*ZRCRAUTC(JI)**2)
#else
            ZHLC_HRCLOCAL(JI)=(3.0*BR_P3(ZHLC_RCMAX(JI)) - 8.0*BR_P3(ZRCRAUTC(JI))) &
                             / (6.0 * BR_P2(ZHLC_RCMAX(JI)) - 12.0*BR_P2(ZRCRAUTC(JI)))
#endif
          END IF
        END IF
      !$mnh_end_do()
!$acc end kernels
    END IF
!$acc kernels present( GWORK, GWORK2 )
    ! Compare r_cM  to r_cR to know if cloud water content is high enough to split in two parts or not
    !$mnh_do_concurrent( JI = 1 : ISIZE )
      GWORK2(JI) = PRCT(JI)>0. .AND. PCF(JI)>0. .AND. ZHLC_RCMAX(JI)<=ZRCRAUTC(JI)
    !$mnh_end_do()
    WHERE(GWORK(:))
      ! Calculate final values for LCF and HCF:
      PHLC_LCF(:)=PCF(:) &
                    *(ZHLC_HRCLOCAL- &
                    (PRCT(:) / PCF(:))) &
                    / (ZHLC_HRCLOCAL-ZHLC_LRCLOCAL)
      PHLC_HCF(:)=MAX(0., PCF(:)-PHLC_LCF(:))
      !
      ! Calculate final values for LRC and HRC:
      PHLC_LRC(:)=ZHLC_LRCLOCAL*PHLC_LCF(:)
      PHLC_HRC(:)=MAX(0., PRCT(:)-PHLC_LRC(:))
    ELSEWHERE (GWORK2(:))
      ! Put all available cloud water and his fraction in the low part
      PHLC_LCF(:)=PCF(:)
      PHLC_HCF(:)=0.
      PHLC_LRC(:)=PRCT(:)
      PHLC_HRC(:)=0.
    ELSEWHERE
      PHLC_LCF(:)=0.
      PHLC_HCF(:)=0.
      PHLC_LRC(:)=0.
      PHLC_HRC(:)=0.
    END WHERE
!$acc end kernels
   ELSE
     CALL PRINT_MSG(NVERB_FATAL,'GEN','ICE4_COMPUTE_PDF','wrong HSUBG_PR_PDF case')
   ENDIF
ELSE
  CALL PRINT_MSG(NVERB_FATAL,'GEN','ICE4_COMPUTE_PDF','wrong HSUBG_AUCV case')
ENDIF
!
!Ice water split between high and low content part is done according to autoconversion option
!$acc kernels present( ZCRIAUTI)
ZCRIAUTI(:)=MIN(XCRIAUTI,10**(XACRIAUTI*(PT(:)-XTT)+XBCRIAUTI)) ! Autoconversion ri threshold
!$acc end kernels
IF(HSUBG_AUCV_RI=='NONE') THEN
!$acc kernels present( ZCRIAUTI)
  !Cloud water is entirely in low or high part
  DO JI = 1, ISIZE
    IF ( PRIT(JI) > ZCRIAUTI(JI) ) THEN
      PHLI_HCF(JI)=1.
      PHLI_LCF(JI)=0.
      PHLI_HRI(JI)=PRIT(JI)
      PHLI_LRI(JI)=0.
    ELSE IF ( PRIT(JI) > XRTMIN(2) ) THEN
      PHLI_HCF(JI)=0.
      PHLI_LCF(JI)=1.
      PHLI_HRI(JI)=0.
      PHLI_LRI(JI)=PRIT(JI)
    ELSE
      PHLI_HCF(JI)=0.
      PHLI_LCF(JI)=0.
      PHLI_HRI(JI)=0.
      PHLI_LRI(JI)=0.
    END IF
  END DO
!$acc end kernels
ELSEIF(HSUBG_AUCV_RI=='CLFR') THEN
!$acc kernels present( ZCRIAUTI)
  !Cloud water is only in the cloudy part and entirely in low or high part
  WHERE(PCF(:)>0. .AND. PRIT(:)>ZCRIAUTI(:)*PCF(:))
    PHLI_HCF(:)=PCF(:)
    PHLI_LCF(:)=0.
    PHLI_HRI(:)=PRIT(:)
    PHLI_LRI(:)=0.
  ELSEWHERE(PCF(:)>0. .AND. PRIT(:)>XRTMIN(2))
    PHLI_HCF(:)=0.
    PHLI_LCF(:)=PCF(:)
    PHLI_HRI(:)=0.0
    PHLI_LRI(:)=PRIT(:)
  ELSEWHERE
    PHLI_HCF(:)=0.
    PHLI_LCF(:)=0.
    PHLI_HRI(:)=0.
    PHLI_LRI(:)=0.
  END WHERE
!$acc end kernels
ELSEIF(HSUBG_AUCV_RI=='ADJU') THEN
!$acc kernels
  ZSUMRI(:)=PHLI_LRI(:)+PHLI_HRI(:)
  WHERE(ZSUMRI .GT. 0.)
    PHLI_LRI(:)=PHLI_LRI(:)*PRIT(:)/ZSUMRI(:)
    PHLI_HRI(:)=PHLI_HRI(:)*PRIT(:)/ZSUMRI(:)
  ELSEWHERE
    PHLI_LRI(:)=0.
    PHLI_HRI(:)=0.
  ENDWHERE
!$acc end kernels
ELSE
  !wrong HSUBG_AUCV_RI case
  CALL PRINT_MSG( NVERB_FATAL, 'GEN', 'ICE4_COMPUTE_PDF', 'wrong HSUBG_AUCV_RI case' )
ENDIF
!
!$acc kernels
PRF=MAX(PHLC_HCF,PHLI_HCF)
!$acc end kernels
!
IF (MPPDB_INITIALIZED) THEN
  !Check all INOUT arrays
  CALL MPPDB_CHECK1D(PHLI_HCF,"ICE4_COMPUTE_PDF beg:PHLI_HCF",PRECISION)
  CALL MPPDB_CHECK1D(PHLI_LCF,"ICE4_COMPUTE_PDF beg:PHLI_LCF",PRECISION)
  CALL MPPDB_CHECK1D(PHLI_HRI,"ICE4_COMPUTE_PDF beg:PHLI_HRI",PRECISION)
  CALL MPPDB_CHECK1D(PHLI_LRI,"ICE4_COMPUTE_PDF beg:PHLI_LRI",PRECISION)
  !Check all OUT arrays
  CALL MPPDB_CHECK1D(PHLC_HCF,"ICE4_COMPUTE_PDF end:PHLC_HCF",PRECISION)
  CALL MPPDB_CHECK1D(PHLC_LCF,"ICE4_COMPUTE_PDF end:PHLC_LCF",PRECISION)
  CALL MPPDB_CHECK1D(PHLC_HRC,"ICE4_COMPUTE_PDF end:PHLC_HRC",PRECISION)
  CALL MPPDB_CHECK1D(PHLC_LRC,"ICE4_COMPUTE_PDF end:PHLC_LRC",PRECISION)
  CALL MPPDB_CHECK1D(PRF,"ICE4_COMPUTE_PDF end:PRF",PRECISION)
END IF

!$acc end data

#ifdef MNH_OPENACC
!Release all memory allocated with MNH_MEM_GET calls since last call to MNH_MEM_POSITION_PIN
CALL MNH_MEM_RELEASE( 'ICE4_COMPUTE_PDF' )
#endif

!$acc end data

END SUBROUTINE ICE4_COMPUTE_PDF

